

#property indicator_separate_window
#property indicator_buffers 7
#property indicator_color1  clrBlue
#property indicator_color2  clrRed
#property indicator_color3  clrDarkBlue
#property indicator_color4  clrMaroon
#property indicator_color5  clrDarkSlateGray
#property indicator_color6  clrRed
#property indicator_color7  clrLime
#property indicator_width1  4
#property indicator_width2  4
#property indicator_width3  4
#property indicator_width4  4
#property indicator_width5  2
#property indicator_width6  2
#property indicator_width7  2
#property strict

//
//
//
//
//

enum maMethod
{
   ma_sma,  // Simple moving average
   ma_ema,  // Exponential moving average
   ma_smma, // Smoothed moving average
   ma_lwma, // Linear weighted moving average
   ma_vwma  // Volume weighted moving average
};
enum stdMethods
{
   std_builtIn, // Built in standard deviation
   std_custSam, // Custom - with sample correction
   std_custNos  // Custom - without sample correction
};

extern int                LRPrd            = 20;               // Linear regression Period
extern int                bolPrd           = 20;               // Bollinger Bands Period
extern maMethod           bolMaMode        = ma_sma;           // Bollinger Average method
extern double             bolDev           = 2.0;              // Bollinger Bands Deviation
extern stdMethods         bolDevType       = std_custNos;      // Deviation calculation type
extern int                keltPrd          = 20;               // Keltner Period
extern double             keltFactor       = 1.5;              // Keltner Spacing between bands
extern ENUM_APPLIED_PRICE AppliedPrice     = PRICE_CLOSE;      // Price to use
extern bool               alertsOn         = true;             // Turn alerts on?
extern bool               alertsOnCurrent  = false;            // Alerts on current (still opened) bar?
extern bool               alertsMessage    = true;             // Alerts showing a pop-up message?
extern bool               alertsPushNotif  = false;            // Alerts sending a push notification?
extern bool               alertsSound      = false;            // Alerts playing sound?
extern bool               alertsEmail      = false;            // Alerts sending email?
extern string             soundFile        = "alert2.wav";     // Alerts Sound File to use
//---- buffers
double lrhuu[];
double lrhud[];
double lrhdd[];
double lrhdu[];
double d[];
double colors[];
double upK[];
double dnK[];
double trend[];


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorBuffers(9);
   SetIndexBuffer(0, lrhuu); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1, lrhdd); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2, lrhud); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3, lrhdu); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4, d); 
   SetIndexBuffer(5, upK);   SetIndexStyle(5,DRAW_ARROW); SetIndexArrow(5,159);
   SetIndexBuffer(6, dnK);   SetIndexStyle(6,DRAW_ARROW); SetIndexArrow(6,159);
   SetIndexBuffer(7, trend);
   SetIndexBuffer(8, colors);
   IndicatorShortName("bbsqueeze("+(string)LRPrd+")");
   return(0);
  }
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = fmin(Bars-counted_bars,Bars-1);
   //
   //
   //
   //
   //
   
   for (i=limit; i>=0; i--)
   {
       double dt=0,vt=0,std;
       if (bolMaMode<ma_vwma)
       {
          dt  = iMA(NULL,0,bolPrd,0,(int)bolMaMode,AppliedPrice,i);
          if (bolDevType==std_builtIn)
                std = iStdDev(NULL,0,bolPrd,0,(int)bolMaMode,AppliedPrice,i);
          else  std = iDeviation(iMA(NULL,0,1,0,0,AppliedPrice,i),bolPrd,i,bolDevType==std_custSam);
       }
       else
       {
          if (bolDevType==std_builtIn)
                std = iStdDev(NULL,0,bolPrd,0,MODE_SMA,AppliedPrice,i);
          else  std = iDeviation(iMA(NULL,0,1,0,0,AppliedPrice,i),bolPrd,i,bolDevType==std_custSam);
          for (int k = 0; k<bolPrd && i+k<Bars; k++) { dt += iMA(NULL,0,1,0,MODE_SMA,AppliedPrice,i+k)*Volume[i+k]; vt += (double)Volume[i+k]; } dt /= vt;
       } 
                  
       d[i] = LinearRegressionSlope(LRPrd,i);
       double atr    = iATR(NULL,0,keltPrd,i);
       double UpMa   = dt + std*bolDev;
       double DnMa   = dt - std*bolDev;
       double keltUp = dt + keltFactor*atr;
       double keltDn = dt - keltFactor*atr;
       lrhuu[i] = EMPTY_VALUE;
       lrhud[i] = EMPTY_VALUE;
       lrhdd[i] = EMPTY_VALUE;
       lrhdu[i] = EMPTY_VALUE;
       upK[i]   = EMPTY_VALUE;
       dnK[i]   = EMPTY_VALUE;
       if (i<Bars-1)
       {
         trend[i]  = trend[i+1];    
         colors[i] = colors[i+1];
            if(UpMa<keltUp && DnMa>keltDn) trend[i] = 1;
            if(UpMa>keltUp && DnMa<keltDn) trend[i] =-1;
            if (trend[i] == 1) upK[i] = 0;
            if (trend[i] ==-1) dnK[i] = 0;
            if (d[i]>d[i+1]) colors[i] =  1;
            if (d[i]<d[i+1]) colors[i] = -1;
            if (d[i]>0)
            if (colors[i]==1)
                  lrhuu[i] = d[i];
            else  lrhud[i] = d[i];
            if (d[i]<0)
            if (colors[i]==1)
                  lrhdu[i] = d[i];
            else  lrhdd[i] = d[i];    
       }
      
   }
   manageAlerts();
return(0);
}

//---------------------------------------------------------------------------------
//
//---------------------------------------------------------------------------------
//
//
//
//
//

int    lrsperiod=EMPTY_VALUE;
double lrssumX;
double lrssumXSqr;
double lrsdivisor;
double LinearRegressionSlope(int len,int shift)
{
   double LinearRegSlope;
   double SumXY = 0;
   double SumY  = 0;

   //
   //
   //
   //
   //

   if (lrsperiod != len)
   {
      lrsperiod  = len;
      lrssumX    = lrsperiod * (lrsperiod-1) / 2;
      lrssumXSqr = lrsperiod * (lrsperiod-1) * (2 * lrsperiod - 1) / 6;
      lrsdivisor = MathPow(lrssumX,2) - lrsperiod * lrssumXSqr;
   }

   //
   //
   //
   //
   //

   for (int i=0; i<lrsperiod; i++)
   {
      double price = iMA(NULL,0,1,0,MODE_EMA,AppliedPrice,i+shift);
            SumXY += i*price;
            SumY  +=   price;
   }
   if( lrsdivisor != 0 ) 
         	LinearRegSlope = (lrsperiod * SumXY - lrssumX * SumY)/lrsdivisor;
   else     LinearRegSlope = 0; 
   return  (LinearRegSlope);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workDev[];
double iDeviation(double value, int length, int i, bool isSample=false)
{
   if (ArraySize(workDev)!=Bars) ArrayResize(workDev,Bars);  i = Bars-i-1; workDev[i] = value;
      double sumx=0,sumxx=0; for (int k=0; k<length && (i-k)>=0; sumx+=workDev[i-k],sumxx+=workDev[i-k]*workDev[i-k],k++) {}
   return(MathSqrt((sumxx-sumx*sumx/length)/fmax(length-isSample,1)));
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar]== 1) doAlert(whichBar,"squeeze");
         if (trend[whichBar]==-1) doAlert(whichBar,"breaking out");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
    static string   previousAlert="nothing";
    static datetime previousTime;
    string message;
   
    if (previousAlert != doWhat || previousTime != Time[forBar]) {
        previousAlert  = doWhat;
        previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," bbsqueeze dark ",doWhat);
             if (alertsMessage)    Alert(message);
             if (alertsPushNotif)  SendNotification(message);
             if (alertsEmail)      SendMail(StringConcatenate(Symbol()," bbsqueeze dark"),message);
             if (alertsSound)      PlaySound(soundFile);
      }
}